#!/usr/bin/env python
#pylint: skip-file
"""
PolicyApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class PolicyApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getFilterPolicies(self, **kwargs):
        """Retrieves policies based on a given filter

        Args:

            policyScope, str: Retrieve policies for a given scope (required)


            scopeWirelessSegment, str: Retrieve policies for a given wireless segment, within a policyScope (policyScope is mandatory for this) (required)


            applicationId, str: Retrieve policies for a given Resource Application Id (required)


            offset, str: Starting index of the resources (1 based) (required)


            limit, str: Number of resources to return (required)



        Returns: PolicyListResult
        """

        allParams = ['policyScope', 'scopeWirelessSegment', 'applicationId', 'offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getFilterPolicies" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('scopeWirelessSegment' in params):
            queryParams['scopeWirelessSegment'] = self.apiClient.toPathValue(params['scopeWirelessSegment'])

        if ('applicationId' in params):
            queryParams['applicationId'] = self.apiClient.toPathValue(params['applicationId'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyListResult')
        return responseObject




    def update(self, **kwargs):
        """Update policy(s)

        Args:

            scheduleAt, str: Epoch Time (The number of milli-seconds since January 1 1970 UTC) at which the policy should be scheduled (Optional)  (required)


            scheduleDesc, str: Custom Description (Optional) (required)


            scheduleOrigin, str: Originator of this call (Optional) (required)


            policyList, list[Policy]: Policy Object (required)



        Returns: TaskIdResult
        """

        allParams = ['scheduleAt', 'scheduleDesc', 'scheduleOrigin', 'policyList']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method update" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scheduleAt' in params):
            queryParams['scheduleAt'] = self.apiClient.toPathValue(params['scheduleAt'])

        if ('scheduleDesc' in params):
            queryParams['scheduleDesc'] = self.apiClient.toPathValue(params['scheduleDesc'])

        if ('scheduleOrigin' in params):
            queryParams['scheduleOrigin'] = self.apiClient.toPathValue(params['scheduleOrigin'])









        if ('policyList' in params):
            bodyParam = params['policyList']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def add(self, **kwargs):
        """Create policy(s)

        Args:

            scheduleAt, str: Epoch Time (The number of milli-seconds since January 1 1970 UTC) at which the policy should be scheduled (Optional)  (required)


            scheduleDesc, str: Custom Description (Optional) (required)


            scheduleOrigin, str: Originator of this call (Optional) (required)


            policyList, list[Policy]: Policy Object (required)



        Returns: TaskIdResult
        """

        allParams = ['scheduleAt', 'scheduleDesc', 'scheduleOrigin', 'policyList']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method add" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scheduleAt' in params):
            queryParams['scheduleAt'] = self.apiClient.toPathValue(params['scheduleAt'])

        if ('scheduleDesc' in params):
            queryParams['scheduleDesc'] = self.apiClient.toPathValue(params['scheduleDesc'])

        if ('scheduleOrigin' in params):
            queryParams['scheduleOrigin'] = self.apiClient.toPathValue(params['scheduleOrigin'])









        if ('policyList' in params):
            bodyParam = params['policyList']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def deleteFilterPolicies(self, **kwargs):
        """Delete policies based on a given filter

        Args:

            policyScope, str: Delete policies for a given scope (required)


            scopeWirelessSegment, str: Delete policies for a given wireless segment, within a policyScope (policyScope is mandatory for this) (required)



        Returns: TaskIdResult
        """

        allParams = ['policyScope', 'scopeWirelessSegment']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteFilterPolicies" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('scopeWirelessSegment' in params):
            queryParams['scopeWirelessSegment'] = self.apiClient.toPathValue(params['scopeWirelessSegment'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getCount(self, **kwargs):
        """Return total count of policies

        Args:


        Returns: CountResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getPolicyDiff(self, **kwargs):
        """Retrieves the policy diff

        Args:

            version, str: Retrieve policy diffs for the given version (required)


            policyScope, str: Retrieve policy diffs for the given scope (required)


            scopeWirelessSegment, str: Retrieve policy diffs for the given scopeWirelessSegment (required)



        Returns: VersionDiffResult
        """

        allParams = ['version', 'policyScope', 'scopeWirelessSegment']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyDiff" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/diff'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('version' in params):
            queryParams['version'] = self.apiClient.toPathValue(params['version'])

        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('scopeWirelessSegment' in params):
            queryParams['scopeWirelessSegment'] = self.apiClient.toPathValue(params['scopeWirelessSegment'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'VersionDiffResult')
        return responseObject




    def addPolicyDiff(self, **kwargs):
        """Adds the policy diff

        Args:

            versionDiffDTO, VersionDiffDTO: VersionDiff (required)



        Returns: TaskIdResult
        """

        allParams = ['versionDiffDTO']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addPolicyDiff" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/diff'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('versionDiffDTO' in params):
            bodyParam = params['versionDiffDTO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getFlows(self, **kwargs):
        """Retrieves flows with search criteria

        Args:

            offset, str: Starting index of the resources (1 based) (required)


            limit, str: Number of resources to return (required)



        Returns: FlowListResult
        """

        allParams = ['offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getFlows" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FlowListResult')
        return responseObject




    def addFlow(self, **kwargs):
        """Creates a new traffic flow for which qos policies will be applied

        Args:

            flowDTO, FlowDTO: flowDTO (required)



        Returns: FlowIdResult
        """

        allParams = ['flowDTO']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addFlow" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('flowDTO' in params):
            bodyParam = params['flowDTO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FlowIdResult')
        return responseObject




    def deleteFlowsByClientReference(self, **kwargs):
        """Deletes flows based on clientReference prefix

        Args:

            clientReference, str: clientReference (required)



        Returns: TaskIdResult
        """

        allParams = ['clientReference']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteFlowsByClientReference" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('clientReference' in params):
            queryParams['clientReference'] = self.apiClient.toPathValue(params['clientReference'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getPolicyFlowSetting(self, **kwargs):
        """Retrieves dynamic policy flow setting

        Args:


        Returns: PolicyFlowSettingResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyFlowSetting" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow/setting'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyFlowSettingResult')
        return responseObject




    def updateFlowPolicySetting(self, **kwargs):
        """Update the dynamic policy flow setting. This setting will be used to allow/disallow dynamic flows /policy/flow requests.

        Args:

            policyFlowSettingDTO, PolicyFlowSettingDTO: policyFlowSettingDTO (required)



        Returns: TaskIdResult
        """

        allParams = ['policyFlowSettingDTO']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateFlowPolicySetting" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow/setting'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('policyFlowSettingDTO' in params):
            bodyParam = params['policyFlowSettingDTO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getFlow(self, **kwargs):
        """Retrieves a flow by its id

        Args:

            id, str: id (required)



        Returns: FlowResult
        """

        allParams = ['id']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getFlow" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FlowResult')
        return responseObject




    def deleteFlow(self, **kwargs):
        """Delete a policy for the flow by its id

        Args:

            id, str: id (required)



        Returns: TaskIdResult
        """

        allParams = ['id']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteFlow" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/flow/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getPolicyStatus(self, **kwargs):
        """Retrieves policy statuses based on a given filter

        Args:

            networkDeviceId, str: Retrieve policies for a given networkDeviceId (required)


            policyScope, str: Retrieve policies for a given scope (required)


            offset, str: Starting index of the resources (1 based) (required)


            limit, str: Number of resources to return (Use smaller limit value for better response times. Max 50) (required)



        Returns: PolicyStatusListResult
        """

        allParams = ['networkDeviceId', 'policyScope', 'offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyStatus" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/status'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('networkDeviceId' in params):
            queryParams['networkDeviceId'] = self.apiClient.toPathValue(params['networkDeviceId'])

        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyStatusListResult')
        return responseObject




    def getCountByPolicyScope(self, **kwargs):
        """Retrieves count of policy status(es) based on a given policyScope &amp; scopeWirelessSegment

        Args:

            policyScope, str: Retrieve count of policy status(es) based on a given policyScope (required)


            scopeWirelessSegment, str: Retrieve count of policy status(es) based on a given scopeWirelessSegment (required)



        Returns: CountResult
        """

        allParams = ['policyScope', 'scopeWirelessSegment']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getCountByPolicyScope" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/status/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('scopeWirelessSegment' in params):
            queryParams['scopeWirelessSegment'] = self.apiClient.toPathValue(params['scopeWirelessSegment'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getPolicyStatusSummary(self, **kwargs):
        """Retrieves policy statuses summary based on a given filter

        Args:

            policyScope, str: Retrieve policy status(es) for a given scope (required)


            offset, str: Starting index of the resources (1 based) (required)


            limit, str: Number of resources to return (Use smaller limit value for better response times. Max 50) (required)



        Returns: PolicyStatusListResult
        """

        allParams = ['policyScope', 'offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyStatusSummary" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/status/summary'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyStatusListResult')
        return responseObject




    def getPolicyTags(self, **kwargs):
        """Retrieves policy tags

        Args:

            policyTag, str: policyTag search substring (required)


            filterOperation, str: type of search (startsWith, endsWith, contains (required)



        Returns: PolicyTagListResult
        """

        allParams = ['policyTag', 'filterOperation']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyTags" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyTag' in params):
            queryParams['policyTag'] = self.apiClient.toPathValue(params['policyTag'])

        if ('filterOperation' in params):
            queryParams['filterOperation'] = self.apiClient.toPathValue(params['filterOperation'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyTagListResult')
        return responseObject




    def addPolicyTag(self, **kwargs):
        """Create a policy tag

        Args:

            policyTagDTO, PolicyTagDTO: policyTagDTO (required)



        Returns: TaskIdResult
        """

        allParams = ['policyTagDTO']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addPolicyTag" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('policyTagDTO' in params):
            bodyParam = params['policyTagDTO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def deletePolicyTag(self, **kwargs):
        """Delete a policy tag.

        Args:

            policyTag, str: Policy Tag (required)



        Returns: TaskIdResult
        """

        allParams = ['policyTag']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deletePolicyTag" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyTag' in params):
            queryParams['policyTag'] = self.apiClient.toPathValue(params['policyTag'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getPolicyTagAssociation(self, **kwargs):
        """Retrieves network device ids that have the policy tag

        Args:

            policyTag, str: Policy Tag (required)


            networkDeviceId, str: Network device Id (required)


            offset, str: Starting index of the resources (1 based) (required)


            limit, str: Number of resources to return (required)



        Returns: PolicyTagAssociationListResult
        """

        allParams = ['policyTag', 'networkDeviceId', 'offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyTagAssociation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag/association'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyTag' in params):
            queryParams['policyTag'] = self.apiClient.toPathValue(params['policyTag'])

        if ('networkDeviceId' in params):
            queryParams['networkDeviceId'] = self.apiClient.toPathValue(params['networkDeviceId'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyTagAssociationListResult')
        return responseObject




    def addPolicyTagAssociation(self, **kwargs):
        """Add a policy tag to network devices.

        Args:

            policyTagAssociationDTO, PolicyTagAssociationDTO: policyTagAssociationDTO (required)



        Returns: TaskIdResult
        """

        allParams = ['policyTagAssociationDTO']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addPolicyTagAssociation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag/association'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('policyTagAssociationDTO' in params):
            bodyParam = params['policyTagAssociationDTO']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def deletePolicyTagAssociation(self, **kwargs):
        """Remove a policy tag from network device.

        Args:

            policyTag, str: Policy Tag (required)


            networkDeviceId, str: NetworkDeviceId (required)



        Returns: TaskIdResult
        """

        allParams = ['policyTag', 'networkDeviceId']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deletePolicyTagAssociation" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag/association'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyTag' in params):
            queryParams['policyTag'] = self.apiClient.toPathValue(params['policyTag'])

        if ('networkDeviceId' in params):
            queryParams['networkDeviceId'] = self.apiClient.toPathValue(params['networkDeviceId'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getPolicyTagsCount(self, **kwargs):
        """Retrieves the number of policy tags

        Args:


        Returns: CountResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyTagsCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/tag/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getPolicyVersionNumbers(self, **kwargs):
        """Retrieves the policy version numbers

        Args:

            policyScope, str: Retrieve policy version numbers for the given scope (required)


            scopeWirelessSegment, str: Retrieve policy version numbers for the given scopeWirelessSegment (required)


            offset, str: Starting index of the resources (1 based) (required)


            limit, str: Number of resources to return (required)



        Returns: PolicyVersionNumberListResult
        """

        allParams = ['policyScope', 'scopeWirelessSegment', 'offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyVersionNumbers" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/version'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('scopeWirelessSegment' in params):
            queryParams['scopeWirelessSegment'] = self.apiClient.toPathValue(params['scopeWirelessSegment'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyVersionNumberListResult')
        return responseObject




    def getPolicyVersionNumbersCount(self, **kwargs):
        """Retrieves the count of policy version numbers

        Args:

            policyScope, str: Retrieve the count of policy version numbers for the given scope (required)


            scopeWirelessSegment, str: Retrieve the count of policy version numbers for the given scopeWirelessSegment (required)



        Returns: CountResult
        """

        allParams = ['policyScope', 'scopeWirelessSegment']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getPolicyVersionNumbersCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/version/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('policyScope' in params):
            queryParams['policyScope'] = self.apiClient.toPathValue(params['policyScope'])

        if ('scopeWirelessSegment' in params):
            queryParams['scopeWirelessSegment'] = self.apiClient.toPathValue(params['scopeWirelessSegment'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def get(self, **kwargs):
        """Retrieves a policy based on its id

        Args:

            id, str: id (required)



        Returns: PolicyResult
        """

        allParams = ['id']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method get" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PolicyResult')
        return responseObject




    def delete(self, **kwargs):
        """Deletes a policy by its id

        Args:

            id, str: id (required)



        Returns: TaskIdResult
        """

        allParams = ['id']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method delete" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/policy/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject









from unittest.mock import MagicMock
import os

import pytest

from uniq.apis.nb.client_manager import NbClientManager
from uniq.apis.exceptions import UniqException


class TestNbClientManager(object):
    """ Tests of nb api client. """

    @pytest.fixture
    def setup_nb_client(self):
        """ Returns faked NbClientManager nb. """

        nb = NbClientManager('1.2.3.4', 'username', 'password', connect=False)
        return nb

    @pytest.fixture
    def setup_nb_client_and_mock(self, setup_nb_client):
        """ Returns faked NbClientManager nb and mocked method 'call_api' """

        nb = setup_nb_client
        mocked_call_api = MagicMock()
        nb.callAPI = mocked_call_api
        return (nb, mocked_call_api)

    def test_v1_only_endpoint_without_api_version(self, setup_nb_client_and_mock):
        """ Verifies nb_api works when calling v1 only endpoints without specifying api version. """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.application.getApplication(id="1")
        mocked_call_api.assert_called_with(
            '/application/1', 'GET', {}, None,
            {'Content-Type': 'application/json', 'Accept': 'application/json'}, files={})

    def test_v1_only_endpoint_with_api_version(self, setup_nb_client_and_mock):
        """ Verifies nb_api works when calling v1 only endpoints with specifying api version. """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.application.getApplication(id="1", api_version='v1')
        mocked_call_api.assert_called_with(
            '/application/1', 'GET', {}, None,
            {'Content-Type': 'application/json', 'Accept': 'application/json'}, files={})

    def test_v1_only_endpoint_raise_exception_when_assigning_v2(self, setup_nb_client_and_mock):
        """ Verifies nb_api raise proper exception when calling v1 only endpoints with specifying
        wrong api version.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock
        with pytest.raises(UniqException) as excinfo:
            nb_api.application.getApplication(id="1", api_version='v2')
        assert (excinfo.value.error_message ==
            "endpoint 'getApplication' doesn't support version 'v2'")

    def test_v2_only_endpoint_without_api_version(self, setup_nb_client_and_mock):
        """ Verifies nb_api works when calling v2 only endpoints without specifying api version. """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.contract.getCount()
        mocked_call_api.assert_called_with(
            '/v2/contract/count', 'GET', {}, None,
            {'Content-Type': 'application/json', 'Accept': 'application/json'}, files={})

    def test_v2_only_endpoint_with_api_version(self, setup_nb_client_and_mock):
        """ Verifies nb_api works when calling v2 only endpoints with specifying api version. """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.contract.getCount(api_version='v2')
        mocked_call_api.assert_called_with(
            '/v2/contract/count', 'GET', {}, None,
            {'Content-Type': 'application/json', 'Accept': 'application/json'}, files={})

    def test_v2_only_endpoint_raise_exception_when_assigning_v1(self, setup_nb_client_and_mock):
        """ Verifies nb_api raise proper exception when calling v2 only endpoints with specifying
        wrong api version.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock
        with pytest.raises(UniqException) as excinfo:
            nb_api.contract.getCount(api_version='v1')
        assert (excinfo.value.error_message ==
            "endpoint 'getCount' doesn't support version 'v1'")

    def test_call_v1_endpoint_when_v1_v2_using_same_name(self, setup_nb_client_and_mock):
        """ Verifies nb_api works when calling v1 endpoints which have both v1 and v2 version and
        their nickname are same.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.add(api_version='v1', scheduleAt='1', scheduleDesc='2',
                          scheduleOrigin='3', policyList={'policy'})
        mocked_call_api.assert_called_with(
            '/policy', 'POST', {'scheduleDesc': '2', 'scheduleOrigin': '3', 'scheduleAt': '1'},
            {'policy'}, {'Accept': 'application/json', 'Content-Type': 'application/json'},
            files={})

    def test_call_v2_endpoint_when_v1_v2_using_same_name(self, setup_nb_client_and_mock):
        """ Verifies nb_api works when calling v2 endpoints which have both v1 and v2 version and
        their nickname are same.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.add(policyDTOs={'new_policy'}, api_version='v2')
        mocked_call_api.assert_called_with(
            '/v2/policy', 'POST', {}, {'new_policy'},
            {'Accept': 'application/json', 'Content-Type': 'application/json'}, files={})

    def test_v1_endpoint_is_default_when_v1_v2_using_same_name(self, setup_nb_client_and_mock):
        """ Verifies v1 endpoints will be called when calling endpoints which have both v1 and v2
        version and their nickname are same.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.add(scheduleAt='1', scheduleDesc='2',
                          scheduleOrigin='3', policyList={'policy'})
        mocked_call_api.assert_called_with(
            '/policy', 'POST', {'scheduleDesc': '2', 'scheduleOrigin': '3', 'scheduleAt': '1'},
            {'policy'}, {'Accept': 'application/json', 'Content-Type': 'application/json'},
            files={})

    def test_call_v1_endpoint_when_v1_v2_using_name_map(self, setup_nb_client_and_mock):
        """ Verifies v1 endpoints can be called properly with v1 original nickname when endpoints
        of v1 and v2 have same url but different nickname.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.get(id='1', api_version='v1')
        mocked_call_api.assert_called_with(
            '/policy/1', 'GET', {}, None,
            {'Accept': 'application/json', 'Content-Type': 'application/json'}, files={})

    def test_call_v2_endpoint_when_v1_v2_using_name_map(self, setup_nb_client_and_mock):
        """ Verifies v1 endpoints can be called properly with v2 original nickname when endpoints
        of v1 and v2 have same url but different nickname.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.getById(id='1', api_version='v2')
        mocked_call_api.assert_called_with(
            '/v2/policy/1', 'GET', {}, None,
            {'Accept': 'application/json', 'Content-Type': 'application/json'}, files={})

    def test_call_v2_endpoint_with_v1_name_when_v1_v2_using_name_map(self,
                                                                     setup_nb_client_and_mock):
        """ Verifies v2 endpoints can be called properly with v1 nickname when endpoints
        of v1 and v2 have same url but different nickname.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.get(id='1', api_version='v2')
        mocked_call_api.assert_called_with(
            '/v2/policy/1', 'GET', {}, None,
            {'Accept': 'application/json', 'Content-Type': 'application/json'}, files={})

    def test_v1_endpoint_is_default_when_v1_v2_using_name_map(self, setup_nb_client_and_mock):
        """ Verifies v1 endpoints can be called properly without specifying api_version when
        endpoints of v1 and v2 have same url but different nickname.
        """

        nb_api, mocked_call_api = setup_nb_client_and_mock

        nb_api.policy.get(id='1')
        mocked_call_api.assert_called_with(
            '/policy/1', 'GET', {}, None,
            {'Accept': 'application/json', 'Content-Type': 'application/json'}, files={})


    @pytest.fixture
    def setup_nb_and_add_faked_models(self, setup_nb_client):
        """ Setup faked NbClientManager and load faked_models for test. """

        nb = setup_nb_client
        nb.add_new_apis(os.path.join(os.path.dirname(os.path.dirname(__file__)), "faked_models"))
        return nb

    def test_two_same_model(self, setup_nb_and_add_faked_models):
        """ Verifies deserialize works if there are two identical models in two clients. """

        nb = setup_nb_and_add_faked_models

        model_obj = nb.deserialize({"name": "name", "id": "id"}, "Same")
        assert model_obj.name == "name" and model_obj.id == "id"

    def test_one_model_contain_another(self, setup_nb_and_add_faked_models):
        """ Verifies deserialize works if there are two models and one is another's subset
        in two clients.
        """

        nb = setup_nb_and_add_faked_models

        model_obj = nb.deserialize({"name": "name", "id": "id"}, "Contain")
        assert model_obj.name == "name" and model_obj.id == "id" and model_obj.contain is None

        model_obj = nb.deserialize({"name": "name", "id": "id", "contain": "contain"}, "Contain")
        assert model_obj.name == "name" and model_obj.id == "id" and model_obj.contain == "contain"

    def test_models_has_diff_attr(self, setup_nb_and_add_faked_models):
        """ Verifies nb client raises proper exception if there are two models have different
        attributes.
        """

        nb = setup_nb_and_add_faked_models
        with pytest.raises(UniqException) as excinfo:
            model_obj = nb.deserialize({"name": "name", "id": "id"}, "DifferentAttr")

        assert excinfo
        assert ("faked_client.models.DifferentAttr.DifferentAttr" in excinfo.value.error_message and
                "other_faked_client.models.DifferentAttr.DifferentAttr"
                in excinfo.value.error_message)

    def test_models_has_diff_type(self, setup_nb_and_add_faked_models):
        """ Verifies nb client raises proper exception if any type of attributes are different
        between two models .
        """

        nb = setup_nb_and_add_faked_models
        with pytest.raises(UniqException) as excinfo:
            model_obj = nb.deserialize({"name": "name", "id": "id"}, "DifferentType")

        assert excinfo
        assert ("faked_client.models.DifferentType.DifferentType" in excinfo.value.error_message and
                "other_faked_client.models.DifferentType.DifferentType"
                in excinfo.value.error_message)


# -*- coding: utf-8 -*-

"""
uniq is a Python API client library for Cisco's Application Policy Infrastructure Controller
Enterprise Module (APIC-EM) Northbound APIs.

*** Description ***

The APIC-EM Northbound Interface is the only API that you will need to control your network
programmatically. The API is function rich and provides you with an easy-to-use, programmatic
control of your network elements, interfaces, and hosts.

The APIC-EM API provides you with the ability to think about your network at a higher policy
level rather than how to implement that policy. When you use the APIC-EM API, your applications
will make network policy decisions, which will then be implemented by the APIC-EM Controller
through its Southbound Interfaces. Thus you tell the network what you want (i.e., the policy)
and the controller figures out how to implement that policy for you.

The APIC-EM API is REST based and thus you will discover and control your network using HTTP
protocol with HTTP verbs (i.e., GET, POST, PUT, and DELETE) with JSON syntax.

This package provides a handle to this rich API library in an easy to consume fashion.

*** Usage ***

Import the package and make an API call.

    >>> from uniq.apis.nb.client_manager import NbClientManager

    >>> client = NbClientManager(
    ...     server="1.1.1.1",
    ...     username="username",
    ...     password="password",
    ...     connect=True)

    >>> # NorthBound API call to get all users
    >>> user_list_result = client.user.getUsers()

    >>> # Serialize the model object to a python dictionary
    >>> users = client.serialize(user_list_result)

    >>> print(users)

:copyright: (c) 2016 by Cisco Systems, see Copyright for more details.
:license: Apache 2.0, see LICENSE for more details.
"""


__title__ = 'uniq'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2016 Cisco Systems'

__version__ = '1.2.0'
__first_release_date__ = '2016.6.8'



#!/usr/bin/env python
#pylint: skip-file
"""
InterfaceApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class InterfaceApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getInterface(self, **kwargs):
        """Retrieves all interfaces

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterface" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfListResult')
        return responseObject




    def getInterfaceCount(self, **kwargs):
        """Retrieves interface count

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getInterfaceListByIp(self, **kwargs):
        """Retrieves interfaces by IP address

        Args:

            ipAddress, str: IP address of the interface (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfListResult
        """

        allParams = ['ipAddress', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceListByIp" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/ip-address/{ipAddress}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('ipAddress' in params):
            replacement = str(self.apiClient.toPathValue(params['ipAddress']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'ipAddress' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfListResult')
        return responseObject




    def getInterfaceByIsis(self, **kwargs):
        """Retrieves ISIS interfaces

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceByIsis" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/isis'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfListResult')
        return responseObject




    def getInterfaceByDeviceId(self, **kwargs):
        """Retrieves device interfaces

        Args:

            deviceId, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfListResult
        """

        allParams = ['deviceId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceByDeviceId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/network-device/{deviceId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('deviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['deviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'deviceId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfListResult')
        return responseObject




    def getInterfaceByDeviceIdAndName(self, **kwargs):
        """Retrieves interface for the given device and interface name

        Args:

            deviceId, str: Device ID (required)


            name, str: Interface name (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfResult
        """

        allParams = ['deviceId', 'name', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceByDeviceIdAndName" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/network-device/{deviceId}/interface-name'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('name' in params):
            queryParams['name'] = self.apiClient.toPathValue(params['name'])



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('deviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['deviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'deviceId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfResult')
        return responseObject




    def getInterfaceRangeByDevice(self, **kwargs):
        """Retrieves interface range for the device

        Args:

            deviceId, str: Device ID (required)


            startIndex, int: Start index (required)


            recordsToReturn, int: Number of records to return (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfListResult
        """

        allParams = ['deviceId', 'startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceRangeByDevice" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/network-device/{deviceId}/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('deviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['deviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'deviceId' + '}',
                                                replacement)

        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfListResult')
        return responseObject




    def getInterfaceCountByDevice(self, **kwargs):
        """Retrieves device interface count

        Args:

            networkDeviceId, str: Device ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['networkDeviceId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceCountByDevice" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/network-device/{networkDeviceId}/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('networkDeviceId' in params):
            replacement = str(self.apiClient.toPathValue(params['networkDeviceId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'networkDeviceId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getInterfaceByOspf(self, **kwargs):
        """Retrieves OSPF interfaces

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceByOspf" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/ospf'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfListResult')
        return responseObject




    def getInterfaceById(self, **kwargs):
        """Retrieves interface by ID

        Args:

            id, str: Interface ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DeviceIfResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/interface/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DeviceIfResult')
        return responseObject




    def getInterfaceByTag(self, **kwargs):
        """Retrieves interfaces by tag

        Args:

            tagId, str: Tag ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: InterfaceNIOListResult
        """

        allParams = ['tagId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceByTag" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/tag/{tagId}/interface'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('tagId' in params):
            replacement = str(self.apiClient.toPathValue(params['tagId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'tagId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'InterfaceNIOListResult')
        return responseObject




    def getInterfaceCountByTagId(self, **kwargs):
        """Retrieves interface count by tag

        Args:

            tagId, str: Tag ID (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['tagId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getInterfaceCountByTagId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/tag/{tagId}/interface/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('tagId' in params):
            replacement = str(self.apiClient.toPathValue(params['tagId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'tagId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject









#!/usr/bin/env python
#pylint: skip-file
"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class NodeWrapper(object):



    def __init__(self):
        """
        Attributes:
          swaggerTypes (dict): The key is attribute name and the value is attribute type.
          attributeMap (dict): The key is attribute name and the value is json key in definition.
        """
        self.swaggerTypes = {

            'role': 'str',


            'order': 'int',


            'id': 'str',


            'label': 'str',


            'roleSource': 'str',


            'customParam': 'NodeWrapperCustom',


            'greyOut': 'bool',


            'deviceType': 'str',


            'ip': 'str',


            'softwareVersion': 'str',


            'nodeType': 'str',


            'family': 'str',


            'platformId': 'str',


            'tags': 'list[str]',


            'aclApplied': 'bool',


            'dataPathId': 'str',


            'upperNode': 'str',


            'fixed': 'bool',


            'osType': 'str',


            'vlanId': 'str',


            'x': 'int',


            'y': 'int',


            'userId': 'str',


            'networkType': 'str'

        }

        self.attributeMap = {

            'role': 'role',

            'order': 'order',

            'id': 'id',

            'label': 'label',

            'roleSource': 'roleSource',

            'customParam': 'customParam',

            'greyOut': 'greyOut',

            'deviceType': 'deviceType',

            'ip': 'ip',

            'softwareVersion': 'softwareVersion',

            'nodeType': 'nodeType',

            'family': 'family',

            'platformId': 'platformId',

            'tags': 'tags',

            'aclApplied': 'aclApplied',

            'dataPathId': 'dataPathId',

            'upperNode': 'upperNode',

            'fixed': 'fixed',

            'osType': 'osType',

            'vlanId': 'vlanId',

            'x': 'x',

            'y': 'y',

            'userId': 'userId',

            'networkType': 'networkType'

        }


        #Role of the device

        self.role = None # str

        #Device order by link number

        self.order = None # int

        #Unique identifier for device

        self.id = None # str

        #Hostname of the device

        self.label = None # str

        #Indicates whether role is assigned manually or automatically

        self.roleSource = None # str

        #Device custom parameters

        self.customParam = None # NodeWrapperCustom

        #Indicates if the device is active for this topology view

        self.greyOut = None # bool

        #Type of the device

        self.deviceType = None # str

        #IP address of the device

        self.ip = None # str

        #Device OS version

        self.softwareVersion = None # str

        #Host or device

        self.nodeType = None # str

        #Product family which is part of the product identifier

        self.family = None # str

        #Device platform description

        self.platformId = None # str

        #List of tags applied on this device

        self.tags = None # list[str]

        #Indicates if the ACL that is applied on the device

        self.aclApplied = None # bool

        #ID of the path between devices

        self.dataPathId = None # str

        #Start node ID

        self.upperNode = None # str

        #Indication of whether the position is fixed or will use auto layout

        self.fixed = None # bool

        #OS type of the device

        self.osType = None # str

        #VLan id

        self.vlanId = None # str

        #X position of the device on the displayed topology view

        self.x = None # int

        #Y position of the device on the displayed topology view

        self.y = None # int

        #ID of the host

        self.userId = None # str

        #Type of network

        self.networkType = None # str




#!/usr/bin/env python
#pylint: skip-file
"""
FlowanalysisApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class FlowanalysisApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getAllFlowAnalysis(self, **kwargs):
        """Retrieves a summary of all flow analyses stored

        Args:

            periodicRefresh, bool: Is analysis periodically refreshed? (required)


            sourceIP, str: Source IP address (required)


            destIP, str: Destination IP adress (required)


            sourcePort, str: Source port (required)


            destPort, str: Destination port (required)


            gtCreateTime, long: Analyses requested after this time (required)


            ltCreateTime, long: Analyses requested before this time (required)


            protocol, str: Protocol (required)


            status, str: Status (required)


            taskId, str: Task ID (required)


            lastUpdateTime, str: Last update time (required)


            limit, str: Number of resources returned (required)


            offset, str: Start index of resources returned (1-based) (required)


            order, str: Order by this field (required)


            sortBy, str: Sort by this field (required)



        Returns: FlowAnalysisListOutput
        """

        allParams = ['periodicRefresh', 'sourceIP', 'destIP', 'sourcePort', 'destPort', 'gtCreateTime', 'ltCreateTime', 'protocol', 'status', 'taskId', 'lastUpdateTime', 'limit', 'offset', 'order', 'sortBy']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getAllFlowAnalysis" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/flow-analysis'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('periodicRefresh' in params):
            queryParams['periodicRefresh'] = self.apiClient.toPathValue(params['periodicRefresh'])

        if ('sourceIP' in params):
            queryParams['sourceIP'] = self.apiClient.toPathValue(params['sourceIP'])

        if ('destIP' in params):
            queryParams['destIP'] = self.apiClient.toPathValue(params['destIP'])

        if ('sourcePort' in params):
            queryParams['sourcePort'] = self.apiClient.toPathValue(params['sourcePort'])

        if ('destPort' in params):
            queryParams['destPort'] = self.apiClient.toPathValue(params['destPort'])

        if ('gtCreateTime' in params):
            queryParams['gtCreateTime'] = self.apiClient.toPathValue(params['gtCreateTime'])

        if ('ltCreateTime' in params):
            queryParams['ltCreateTime'] = self.apiClient.toPathValue(params['ltCreateTime'])

        if ('protocol' in params):
            queryParams['protocol'] = self.apiClient.toPathValue(params['protocol'])

        if ('status' in params):
            queryParams['status'] = self.apiClient.toPathValue(params['status'])

        if ('taskId' in params):
            queryParams['taskId'] = self.apiClient.toPathValue(params['taskId'])

        if ('lastUpdateTime' in params):
            queryParams['lastUpdateTime'] = self.apiClient.toPathValue(params['lastUpdateTime'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('order' in params):
            queryParams['order'] = self.apiClient.toPathValue(params['order'])

        if ('sortBy' in params):
            queryParams['sortBy'] = self.apiClient.toPathValue(params['sortBy'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FlowAnalysisListOutput')
        return responseObject




    def initiateFlowAnalysis(self, **kwargs):
        """Initiates a new flow analysis

        Args:

            flowAnalysisRequest, FlowAnalysisRequest: flowAnalysisRequest (required)



        Returns: FlowAnalysisRequestResultOutput
        """

        allParams = ['flowAnalysisRequest']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method initiateFlowAnalysis" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/flow-analysis'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'










        if ('flowAnalysisRequest' in params):
            bodyParam = params['flowAnalysisRequest']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FlowAnalysisRequestResultOutput')
        return responseObject




    def getFullFlowAnalysisResult(self, **kwargs):
        """Retrieves result of a previously requested flow analysis

        Args:

            flowAnalysisId, str: Flow analysis request id (required)



        Returns: PathResponseResult
        """

        allParams = ['flowAnalysisId']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getFullFlowAnalysisResult" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/flow-analysis/{flowAnalysisId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('flowAnalysisId' in params):
            replacement = str(self.apiClient.toPathValue(params['flowAnalysisId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'flowAnalysisId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'PathResponseResult')
        return responseObject




    def deleteFlowAnalysis(self, **kwargs):
        """Deletes a flow analysis request

        Args:

            flowAnalysisId, str: Flow analysis request id (required)



        Returns: TaskIdResult
        """

        allParams = ['flowAnalysisId']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteFlowAnalysis" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/flow-analysis/{flowAnalysisId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('flowAnalysisId' in params):
            replacement = str(self.apiClient.toPathValue(params['flowAnalysisId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'flowAnalysisId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject









#!/usr/bin/env python
#pylint: skip-file
"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class ApplicationDTO(object):



    def __init__(self):
        """
        Attributes:
          swaggerTypes (dict): The key is attribute name and the value is attribute type.
          attributeMap (dict): The key is attribute name and the value is json key in definition.
        """
        self.swaggerTypes = {

            'name': 'str',


            'id': 'str',


            'trafficClass': 'str',


            'rank': 'int',


            'status': 'str',


            'categoryId': 'str',


            'instanceUuid': 'str',


            'subCategory': 'str',


            'globalId': 'str',


            'engineId': 'str',


            'selectorId': 'str',


            'helpString': 'str',


            'references': 'str',


            'applicationGroup': 'str',


            'encrypted': 'str',


            'tunnel': 'str',


            'category': 'str',


            'longDescription': 'str',


            'appProtocol': 'str',


            'tcpPorts': 'str',


            'indicativeTcpPorts': 'str',


            'udpPorts': 'str',


            'indicativeUdpPorts': 'str',


            'ipPorts': 'str',


            'url': 'str',


            'enabled': 'str',


            'indicativeIpPorts': 'str',


            'isRepresentativeApp': 'bool',


            'nbarId': 'str',


            'p2pTechnology': 'str',


            'pfrThresholdJitterPriority': 'int',


            'pfrThresholdLossRatePriority': 'int',


            'pfrThresholdOneWayDelayPriority': 'int',


            'dscp': 'str',


            'pfrThresholdJitter': 'int',


            'pfrThresholdLossRate': 'int',


            'pfrThresholdOneWayDelay': 'int',


            'popularity': 'int',


            'transportIps': 'str',


            'ignoreConflict': 'bool'

        }

        self.attributeMap = {

            'name': 'name',

            'id': 'id',

            'trafficClass': 'trafficClass',

            'rank': 'rank',

            'status': 'status',

            'categoryId': 'categoryId',

            'instanceUuid': 'instanceUuid',

            'subCategory': 'subCategory',

            'globalId': 'globalId',

            'engineId': 'engineId',

            'selectorId': 'selectorId',

            'helpString': 'helpString',

            'references': 'references',

            'applicationGroup': 'applicationGroup',

            'encrypted': 'encrypted',

            'tunnel': 'tunnel',

            'category': 'category',

            'longDescription': 'longDescription',

            'appProtocol': 'appProtocol',

            'tcpPorts': 'tcpPorts',

            'indicativeTcpPorts': 'indicativeTcpPorts',

            'udpPorts': 'udpPorts',

            'indicativeUdpPorts': 'indicativeUdpPorts',

            'ipPorts': 'ipPorts',

            'url': 'url',

            'enabled': 'enabled',

            'indicativeIpPorts': 'indicativeIpPorts',

            'isRepresentativeApp': 'isRepresentativeApp',

            'nbarId': 'nbarId',

            'p2pTechnology': 'p2pTechnology',

            'pfrThresholdJitterPriority': 'pfrThresholdJitterPriority',

            'pfrThresholdLossRatePriority': 'pfrThresholdLossRatePriority',

            'pfrThresholdOneWayDelayPriority': 'pfrThresholdOneWayDelayPriority',

            'dscp': 'dscp',

            'pfrThresholdJitter': 'pfrThresholdJitter',

            'pfrThresholdLossRate': 'pfrThresholdLossRate',

            'pfrThresholdOneWayDelay': 'pfrThresholdOneWayDelay',

            'popularity': 'popularity',

            'transportIps': 'transportIps',

            'ignoreConflict': 'ignoreConflict'

        }


        #App Name

        self.name = None # str

        #id

        self.id = None # str

        #Traffic class to which the app belongs

        self.trafficClass = None # str

        #rank

        self.rank = None # int

        #Gives status of the app

        self.status = None # str

        #Category id

        self.categoryId = None # str

        #

        self.instanceUuid = None # str

        #Sub-Category Id

        self.subCategory = None # str

        #global id

        self.globalId = None # str

        #engine id

        self.engineId = None # str

        #selector id

        self.selectorId = None # str

        #help string to describe the app

        self.helpString = None # str

        #references of the app

        self.references = None # str

        #App group name

        self.applicationGroup = None # str

        #If the app is encrypted

        self.encrypted = None # str

        #If the app is a tunnel

        self.tunnel = None # str

        #Category name

        self.category = None # str

        #Long description of the app

        self.longDescription = None # str

        #protocol of the app. Valid values are tcp, udp, tcp/udp, ip or it could be empty. Values are case sensitive.

        self.appProtocol = None # str

        #list of tcp ports

        self.tcpPorts = None # str

        #Indicative tcp ports

        self.indicativeTcpPorts = None # str

        #list of udp ports

        self.udpPorts = None # str

        #Indicative udp ports

        self.indicativeUdpPorts = None # str

        #list of ip ports

        self.ipPorts = None # str

        #url of the app

        self.url = None # str

        #If the app enabled

        self.enabled = None # str

        #Indicative ip ports

        self.indicativeIpPorts = None # str

        #If the app is representative

        self.isRepresentativeApp = None # bool

        #nbar id

        self.nbarId = None # str

        #If the app is a p2p technology

        self.p2pTechnology = None # str

        #PfR Threshold Jitter Priority

        self.pfrThresholdJitterPriority = None # int

        #PfR Threshold Loss Rate Priority

        self.pfrThresholdLossRatePriority = None # int

        #PfR Threshold One Way Delay Priority

        self.pfrThresholdOneWayDelayPriority = None # int

        #dscp value

        self.dscp = None # str

        #PfR Threshold Jitter

        self.pfrThresholdJitter = None # int

        #PfR Threshold Loss Rate

        self.pfrThresholdLossRate = None # int

        #PfR Threshold One Way Delay

        self.pfrThresholdOneWayDelay = None # int

        #popularity of the app

        self.popularity = None # int

        #Transport IP of the app

        self.transportIps = None # str

        #If true ignore conflicts with other Applications

        self.ignoreConflict = None # bool




#!/usr/bin/env python
#pylint: skip-file
"""
FileApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class FileApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def uploadFileWithInternalUploadUrl(self, **kwargs):
        """uploadFileWithInternalUploadUrl

        Args:

            internalUploadId, str: internalUploadId (required)


            fileUpload, :  (required)



        Returns: FileObjectResult
        """

        allParams = ['internalUploadId', 'fileUpload']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method uploadFileWithInternalUploadUrl" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/internalupload/{internalUploadId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('internalUploadId' in params):
            replacement = str(self.apiClient.toPathValue(params['internalUploadId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'internalUploadId' + '}',
                                                replacement)





        if ('fileUpload' in params):
            bodyParam = params['fileUpload']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FileObjectResult')
        return responseObject




    def getNameSpaceList(self, **kwargs):
        """getNameSpaceList

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: NameSpaceListResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNameSpaceList" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/namespace'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NameSpaceListResult')
        return responseObject




    def getFilesByNamespace(self, **kwargs):
        """getFilesByNamespace

        Args:

            nameSpace, str: A listing of fileId&#39;s (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: FileObjectListResult
        """

        allParams = ['nameSpace', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getFilesByNamespace" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/namespace/{nameSpace}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = ''
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('nameSpace' in params):
            replacement = str(self.apiClient.toPathValue(params['nameSpace']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'nameSpace' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FileObjectListResult')
        return responseObject




    def downloadFileWithOneTimeDownloadUrl(self, **kwargs):
        """downloadFileWithOneTimeDownloadUrl

        Args:

            onetimeDownloadId, str: URL pointing to the location of the file (required)



        Returns: object
        """

        allParams = ['onetimeDownloadId']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method downloadFileWithOneTimeDownloadUrl" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/onetimedownload/{onetimeDownloadId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/octet-stream'
        headerParams['Content-Type'] = 'application/json'






        if ('onetimeDownloadId' in params):
            replacement = str(self.apiClient.toPathValue(params['onetimeDownloadId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'onetimeDownloadId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'object')
        return responseObject




    def downloadFileWithTemporaryDownloadUrl(self, **kwargs):
        """Download file via timedDownloadId.

        Args:

            temporaryDownloadId, str: URL pointing to the location of the file (required)



        Returns: object
        """

        allParams = ['temporaryDownloadId']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method downloadFileWithTemporaryDownloadUrl" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/temporary/{temporaryDownloadId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/octet-stream'
        headerParams['Content-Type'] = 'application/json'






        if ('temporaryDownloadId' in params):
            replacement = str(self.apiClient.toPathValue(params['temporaryDownloadId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'temporaryDownloadId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'object')
        return responseObject




    def downLoadFile(self, **kwargs):
        """downloadFile

        Notes:
            This method has been modified. A parameter 'stream' is added to support the scenario
            that same api is called with different headers.


        Args:

            fileId, str: File Identification number (required)


            scope, str: Authorization Scope for RBAC (required)


            stream, bool: Return stream object.



        Returns: File content
        """

        allParams = ['fileId', 'scope', 'stream']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method downLoadFile" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/{fileId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('fileId' in params):
            replacement = str(self.apiClient.toPathValue(params['fileId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'fileId' + '}',
                                                replacement)

        stream = False
        if ('stream' in params) and params['stream']:
            headerParams['Accept'] = 'application/octet-stream'
            headerParams['Content-Type'] = 'application/octet-stream'
            stream = params['stream']

        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files, stream=stream)


        if not response:
            return None

        #responseObject = self.apiClient.deserialize(response, 'object')
        #return responseObject

        return response

    def deleteFile(self, **kwargs):
        """deleteFile

        Args:

            fileId, str: File Identification number (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: SuccessResult
        """

        allParams = ['fileId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteFile" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/{fileId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('fileId' in params):
            replacement = str(self.apiClient.toPathValue(params['fileId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'fileId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'SuccessResult')
        return responseObject




    def getChecksumOfFile(self, **kwargs):
        """getChecksumOfFile

        Args:

            fileId, str: File Identification number (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: SuccessResult
        """

        allParams = ['fileId', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getChecksumOfFile" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/{fileId}/checksum'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('fileId' in params):
            replacement = str(self.apiClient.toPathValue(params['fileId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'fileId' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'SuccessResult')
        return responseObject




    def uploadFile(self, **kwargs):
        """uploadFile

        Args:

            nameSpace, str: Specify File&#39;s namespace,namespace is a grouping of multiple files (required)


            scope, str: Authorization Scope for RBAC (required)


            fileUpload, :  (required)



        Returns: FileObjectResult
        """

        allParams = ['nameSpace', 'scope', 'fileUpload']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method uploadFile" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/{nameSpace}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'multipart/form-data'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('nameSpace' in params):
            replacement = str(self.apiClient.toPathValue(params['nameSpace']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'nameSpace' + '}',
                                                replacement)





        if ('fileUpload' in params):
            bodyParam = params['fileUpload']


        postData = (formParams if formParams else bodyParam)
        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FileObjectResult')
        return responseObject




    def updateFile(self, **kwargs):
        """updateFile

        Args:

            nameSpace, str: Specify File&#39;s namespace,namespace is a grouping of multiple files (required)


            fileId, str: Specify File Identification number (required)


            scope, str: Authorization Scope for RBAC (required)


            fileUpload, :  (required)



        Returns: FileObjectResult
        """

        allParams = ['nameSpace', 'fileId', 'scope', 'fileUpload']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateFile" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/file/{nameSpace}/{fileId}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'multipart/form-data'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('nameSpace' in params):
            replacement = str(self.apiClient.toPathValue(params['nameSpace']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'nameSpace' + '}',
                                                replacement)

        if ('fileId' in params):
            replacement = str(self.apiClient.toPathValue(params['fileId']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'fileId' + '}',
                                                replacement)





        if ('fileUpload' in params):
            bodyParam = params['fileUpload']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'FileObjectResult')
        return responseObject









#!/usr/bin/env python
#pylint: skip-file
"""
DiscoveryApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class DiscoveryApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def updateDiscovery(self, **kwargs):
        """Updates an existing discovery specified by id - only for starting/stopping the discovery

        Args:

            discovery, DiscoveryNIO: discovery (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['discovery', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateDiscovery" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('discovery' in params):
            bodyParam = params['discovery']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def insertDiscovery(self, **kwargs):
        """Starts a new discovery process and returns a task-id

        Args:

            request, InventoryRequest: request (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['request', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method insertDiscovery" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']







        if ('request' in params):
            bodyParam = params['request']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def deleteAllDiscovery(self, **kwargs):
        """Deletes all discovery

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteAllDiscovery" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getDiscoveryCount(self, **kwargs):
        """Returns the number of discovery

        Args:

            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getDiscoveryCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']








        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getDiscoveryById(self, **kwargs):
        """Returns the discovery specified by id

        Args:

            id, str: id (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DiscoveryNIOResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getDiscoveryById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DiscoveryNIOResult')
        return responseObject




    def deleteDiscoveryById(self, **kwargs):
        """Deletes the discovery specified by id

        Args:

            id, str: id (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteDiscoveryById" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getNetworkDeviceByDiscoveryId(self, **kwargs):
        """Returns the network devices discovered in the discovery specified by id

        Args:

            id, str: id (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceNIOListResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceByDiscoveryId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{id}/network-device'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'




        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceNIOListResult')
        return responseObject




    def getNetworkDeviceCountByDiscoveryId(self, **kwargs):
        """Returns the number of network devices discovered in the discovery specified by id

        Args:

            id, str: id (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: CountResult
        """

        allParams = ['id', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceCountByDiscoveryId" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{id}/network-device/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getNetworkDeviceByDiscoveryIdByRange(self, **kwargs):
        """Returns the network devices discovered in the given range

        Args:

            id, str: id (required)


            startIndex, int: startIndex (required)


            recordsToReturn, int: recordsToReturn (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: NetworkDeviceNIOListResult
        """

        allParams = ['id', 'startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getNetworkDeviceByDiscoveryIdByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{id}/network-device/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)

        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'NetworkDeviceNIOListResult')
        return responseObject




    def deleteDiscoveryByRange(self, **kwargs):
        """Deletes the discovery in the given range

        Args:

            startIndex, int: startIndex (required)


            recordsToDelete, int: recordsToDelete (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: TaskIdResult
        """

        allParams = ['startIndex', 'recordsToDelete', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteDiscoveryByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{startIndex}/{recordsToDelete}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToDelete' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToDelete']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToDelete' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getDiscoveryByRange(self, **kwargs):
        """Returns the discovery in the given range

        Args:

            startIndex, int: startIndex (required)


            recordsToReturn, int: recordsToReturn (required)


            scope, str: Authorization Scope for RBAC (required)



        Returns: DiscoveryNIOListResult
        """

        allParams = ['startIndex', 'recordsToReturn', 'scope']

        params = locals()
        for (key, val) in params['kwargs'].items():
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getDiscoveryByRange" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/discovery/{startIndex}/{recordsToReturn}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'



        if ('scope' in params):
            headerParams['scope'] = params['scope']



        if ('startIndex' in params):
            replacement = str(self.apiClient.toPathValue(params['startIndex']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'startIndex' + '}',
                                                replacement)

        if ('recordsToReturn' in params):
            replacement = str(self.apiClient.toPathValue(params['recordsToReturn']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'recordsToReturn' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'DiscoveryNIOListResult')
        return responseObject









#!/usr/bin/env python
#pylint: skip-file
"""
RelevanceApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class RelevanceApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getAllRelevanceLevels(self, **kwargs):
        """Lists all relevances

        Args:


        Returns: RelevanceListResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getAllRelevanceLevels" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/relevance'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'RelevanceListResult')
        return responseObject









"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import time

try:
    from uniq.common import logger
    logger = logger.log
except:
    import logging
    logger = logging.getLogger("uniq")


class Wait(object):
    """ Utility to wait. """

    TIMEOUT = 60
    INTERVAL = 1
    IGNORED_EXCEPTIONS = tuple()

    def __init__(self, timeout=TIMEOUT, interval=INTERVAL, ignored_exceptions=IGNORED_EXCEPTIONS):
        """ Initializer of Wait Util.

        Args:
            timeout (int): Number of seconds before timing out.
            interval (int): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
        """

        self.log = logger
        self._timeout = timeout
        self._interval = interval
        if type(ignored_exceptions) is list:
            ignored_exceptions = tuple(ignored_exceptions)

        self._ignored_exceptions = ignored_exceptions

    def until(self, method, timeout=None, interval=None, ignored_exceptions=None, message='',
              *args, **kwargs):
        """ Calls the method provided until the return value is not False.

        Args:
            method (function): method to call for checking status.
            timeout (int): Number of seconds before timing out.
            interval (int): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
            message (str): Message shows in TimeoutException.
            args (list): Arguments of method.
            kwargs (dict): Dictionary of arguments of method.

        Returns:
            Value of method if value is not False.

        Raises:
            TimeoutException if value of method is still False when time's up.
        """

        return self._until(method, False, timeout, interval, ignored_exceptions, message,
                           *args, **kwargs)

    def until_not(self, method, timeout=None, interval=None, ignored_exceptions=None, message='',
                  *args, **kwargs):
        """ Calls the method provided until the return value is False.

        Args:
            method (function): method to call for checking status.
            timeout (int): Number of seconds before timing out.
            interval (int): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
            message (str): Message shows in TimeoutException.
            args (list): Arguments of method.
            kwargs (dict): Dictionary of arguments of method.

        Returns:
            False if value of method turns to False in time.

        Raises:
            TimeoutException if value of method is not False when time's up.
        """

        self._until(method, True, timeout, interval, ignored_exceptions, message,
                    *args, **kwargs)
        return False

    def _until(self, method, wait_false=False, timeout=None, interval=None, ignored_exceptions=None,
               message='', *args, **kwargs):
        """ Calls the method provided until the return value is not False.

        Args:
            method (function): method to call for checking status.
            wait_false (bool): true if wait method return value is not False, otherwise, wait return
                               value become False.
            timeout (int): Number of seconds before timing out.
            interval (int): Sleep interval between calls.
            ignored_exceptions (list(Exception)): Exceptions to ignore.
            message (str): Message shows in TimeoutException.
            args (list): Arguments of method.
            kwargs (dict): Dictionary of arguments of method.

        Returns:
            Value of method if value is not False. False if timeout.

        Raises:
            TimeoutException if value of method is still False when time's up.
        """

        if timeout is None:
            timeout = self._timeout
        if interval is None:
            interval = self._interval
        if ignored_exceptions is None:
            ignored_exceptions = self._ignored_exceptions
        else:
            ignored_exceptions = tuple(ignored_exceptions)

        start_time = time.time()
        self.log.debug("Begins at {0}".format(start_time))

        end_time = start_time + timeout
        while True:
            try:
                value = method(*args, **kwargs)
                if wait_false:
                    value = not value
                if value is not False:
                    self.log.debug("Waited for {0} seconds.".format(time.time() - start_time))
                    return value
            except ignored_exceptions as exc:
                self.log.debug("Ignore exception: {0}:{1}".format(type(exc), exc))
            except:
                raise
            time.sleep(interval)
            if time.time() > end_time:
                break
        raise TimeoutException(message)

class TimeoutException(Exception):
    pass





#!/usr/bin/env python
#pylint: skip-file
"""
Copyright 2016 Cisco Systems

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

class ZtdSite(object):



    def __init__(self):
        """
        Attributes:
          swaggerTypes (dict): The key is attribute name and the value is attribute type.
          attributeMap (dict): The key is attribute name and the value is json key in definition.
        """
        self.swaggerTypes = {

            'state': 'str',


            'id': 'str',


            'provisionedBy': 'str',


            'provisionedOn': 'str',


            'siteName': 'str',


            'tftpServer': 'str',


            'tftpPath': 'str',


            'note': 'str',


            'deviceCount': 'int',


            'pendingDeviceCount': 'int',


            'deviceLastUpdate': 'str',


            'installerUserID': 'str'

        }

        self.attributeMap = {

            'state': 'state',

            'id': 'id',

            'provisionedBy': 'provisionedBy',

            'provisionedOn': 'provisionedOn',

            'siteName': 'siteName',

            'tftpServer': 'tftpServer',

            'tftpPath': 'tftpPath',

            'note': 'note',

            'deviceCount': 'deviceCount',

            'pendingDeviceCount': 'pendingDeviceCount',

            'deviceLastUpdate': 'deviceLastUpdate',

            'installerUserID': 'installerUserID'

        }


        #Project state

        self.state = None # str

        #Project ID

        self.id = None # str

        #User creating the project

        self.provisionedBy = None # str

        #Creation time for project

        self.provisionedOn = None # str

        #Project name

        self.siteName = None # str

        #TFTP server host name or IP address

        self.tftpServer = None # str

        #TFTP server path

        self.tftpPath = None # str

        #Project notes. Any file can be attached

        self.note = None # str

        #Number of devices under the project

        self.deviceCount = None # int

        #Number of devices in pending state

        self.pendingDeviceCount = None # int

        #Last contact time among all devices in this project

        self.deviceLastUpdate = None # str

        #Installer user ID

        self.installerUserID = None # str




#!/usr/bin/env python
#pylint: skip-file
"""
ApplicationApi.py
    Copyright 2016 Cisco Systems

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

"""
import sys
import os
import urllib.request, urllib.parse, urllib.error

from .models import *


class ApplicationApi(object):

    def __init__(self, apiClient):
      self.apiClient = apiClient



    def getFilterApplication(self, **kwargs):
        """Retrieves applications based on a given filter, filters should be applied one at a time, except for the limit/index pagination filters.

        Args:

            isCustom, str: Retrieve custom applications (required)


            isRepresentative, str: Retrieve representative applications (required)


            categoryId, str: Retrieve applications by categoryId (required)


            name, str: name (required)


            offset, str: Starting index of the resources (1 based), This should be only used in conjuction with the limit param. (required)


            limit, str: Number of resources to return, WARNING: results may take an unexpectely long time with more than 50 results requested. (required)



        Returns: ApplicationListResult
        """

        allParams = ['isCustom', 'isRepresentative', 'categoryId', 'name', 'offset', 'limit']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getFilterApplication" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/application'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('isCustom' in params):
            queryParams['isCustom'] = self.apiClient.toPathValue(params['isCustom'])

        if ('isRepresentative' in params):
            queryParams['isRepresentative'] = self.apiClient.toPathValue(params['isRepresentative'])

        if ('categoryId' in params):
            queryParams['categoryId'] = self.apiClient.toPathValue(params['categoryId'])

        if ('name' in params):
            queryParams['name'] = self.apiClient.toPathValue(params['name'])

        if ('offset' in params):
            queryParams['offset'] = self.apiClient.toPathValue(params['offset'])

        if ('limit' in params):
            queryParams['limit'] = self.apiClient.toPathValue(params['limit'])










        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'ApplicationListResult')
        return responseObject




    def updateApplication(self, **kwargs):
        """Updates application(s)

        Args:

            scheduleAt, str: scheduleAt (required)


            scheduleDesc, str: scheduleDesc (required)


            scheduleOrigin, str: scheduleOrigin (required)


            applicationDTOList, list[ApplicationDTO]: applicationDTOList (required)



        Returns: TaskIdResult
        """

        allParams = ['scheduleAt', 'scheduleDesc', 'scheduleOrigin', 'applicationDTOList']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method updateApplication" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/application'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'PUT'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scheduleAt' in params):
            queryParams['scheduleAt'] = self.apiClient.toPathValue(params['scheduleAt'])

        if ('scheduleDesc' in params):
            queryParams['scheduleDesc'] = self.apiClient.toPathValue(params['scheduleDesc'])

        if ('scheduleOrigin' in params):
            queryParams['scheduleOrigin'] = self.apiClient.toPathValue(params['scheduleOrigin'])









        if ('applicationDTOList' in params):
            bodyParam = params['applicationDTOList']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def addApplication(self, **kwargs):
        """Creates custom application(s)

        Args:

            scheduleAt, str: scheduleAt (required)


            scheduleDesc, str: scheduleDesc (required)


            scheduleOrigin, str: scheduleOrigin (required)


            applicationDTOList, list[ApplicationDTO]: applicationDTOList (required)



        Returns: TaskIdResult
        """

        allParams = ['scheduleAt', 'scheduleDesc', 'scheduleOrigin', 'applicationDTOList']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method addApplication" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/application'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'POST'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scheduleAt' in params):
            queryParams['scheduleAt'] = self.apiClient.toPathValue(params['scheduleAt'])

        if ('scheduleDesc' in params):
            queryParams['scheduleDesc'] = self.apiClient.toPathValue(params['scheduleDesc'])

        if ('scheduleOrigin' in params):
            queryParams['scheduleOrigin'] = self.apiClient.toPathValue(params['scheduleOrigin'])









        if ('applicationDTOList' in params):
            bodyParam = params['applicationDTOList']


        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject




    def getApplicationCount(self, **kwargs):
        """Return total count of application(s)

        Args:


        Returns: CountResult
        """

        allParams = []

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getApplicationCount" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/application/count'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'











        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'CountResult')
        return responseObject




    def getApplication(self, **kwargs):
        """Return an application given an id

        Args:

            id, str: id (required)



        Returns: ApplicationResult
        """

        allParams = ['id']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method getApplication" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/application/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'GET'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'






        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'ApplicationResult')
        return responseObject




    def deleteApplication(self, **kwargs):
        """Delete an application by id

        Args:

            scheduleAt, str: scheduleAt (required)


            scheduleDesc, str: scheduleDesc (required)


            scheduleOrigin, str: scheduleOrigin (required)


            id, str: id (required)



        Returns: TaskIdResult
        """

        allParams = ['scheduleAt', 'scheduleDesc', 'scheduleOrigin', 'id']

        params = locals()
        for (key, val) in list(params['kwargs'].items()):
            if key not in allParams:
                raise TypeError("Got an unexpected keyword argument '%s' to method deleteApplication" % key)
            params[key] = val
        del params['kwargs']

        resourcePath = '/application/{id}'
        resourcePath = resourcePath.replace('{format}', 'json')
        method = 'DELETE'

        queryParams = {}
        headerParams = {}
        formParams = {}
        files = {}
        bodyParam = None

        headerParams['Accept'] = 'application/json'
        headerParams['Content-Type'] = 'application/json'


        if ('scheduleAt' in params):
            queryParams['scheduleAt'] = self.apiClient.toPathValue(params['scheduleAt'])

        if ('scheduleDesc' in params):
            queryParams['scheduleDesc'] = self.apiClient.toPathValue(params['scheduleDesc'])

        if ('scheduleOrigin' in params):
            queryParams['scheduleOrigin'] = self.apiClient.toPathValue(params['scheduleOrigin'])





        if ('id' in params):
            replacement = str(self.apiClient.toPathValue(params['id']))
            replacement = urllib.parse.quote(replacement)
            resourcePath = resourcePath.replace('{' + 'id' + '}',
                                                replacement)






        postData = (formParams if formParams else bodyParam)

        response = self.apiClient.callAPI(resourcePath, method, queryParams,
                                          postData, headerParams, files=files)


        if not response:
            return None

        responseObject = self.apiClient.deserialize(response, 'TaskIdResult')
        return responseObject






